from __future__ import annotations

from collections.abc import Callable
from typing import TYPE_CHECKING

import numpy as np

from avidaq import ExampleQWidget

if TYPE_CHECKING:
    from _pytest.capture import CaptureFixture
    from napari import Viewer

    ViewerFactory = Callable[[], Viewer]


# make_napari_viewer is a pytest fixture that returns a napari viewer object
# capsys is a pytest fixture that captures stdout and stderr output streams
def test_example_q_widget(
    make_napari_viewer: ViewerFactory, capsys: CaptureFixture
) -> None:
    # make viewer and add an image layer using our fixture
    viewer = make_napari_viewer()
    viewer.add_image(np.random.random((100, 100)))

    # create our widget, passing in the viewer
    my_widget = ExampleQWidget(viewer)

    # call our widget method
    my_widget.receive_data(np.random.random((100, 100)))

    # read captured output and check that it's as we expected
    captured = capsys.readouterr()
    assert captured.out == "napari has 2 layers\n"



""" This module contains functions for dealing with image data."""
from __future__ import annotations

import shutil
from collections.abc import Iterator
from pathlib import Path

__all__ = ["get_image_directories", "delete_image_data"]


def get_image_directories(*, base_path: Path, name: str) -> Iterator[Path]:
    """
    Args:
        base_path (Path): Where to start looking for image directories.
        target (str): The name that was given to micromanager to create
        the image directories.

    Yields:
        Iterator[Path]: directories of where a named image is stored
    """
    return (
        entry
        for entry in base_path.glob("*")
        if name in entry.stem and entry.is_dir()
    )


def delete_image_data(image_directories: Iterator[Path]) -> None:
    for directory in image_directories:
        shutil.rmtree(directory)



from __future__ import annotations

import json
from dataclasses import dataclass
from pathlib import Path
from typing import TypedDict

import numpy.typing as npt

__all__ = ["IMDAParams", "MDAPresets", "get_mda_presets", "MDA"]


class IMDAParams(TypedDict, total=False):
    """Inputs for the multi-dimensional acquisition events."""

    num_time_points: int
    time_interval_s: float
    z_start: float
    z_end: float
    z_step: float
    channel_group: str
    channels: list
    channel_exposures_ms: list
    xy_positions: npt.ArrayLike
    xyz_positions: npt.ArrayLike
    order: str
    keep_shutter_open_between_channels: bool
    keep_shutter_open_between_z_steps: bool


class MDAPresets(TypedDict):
    Default: IMDAParams
    Simple: IMDAParams
    Detailed: IMDAParams


def get_mda_presets() -> MDAPresets:
    return MDAPresets(
        Default=IMDAParams(
            num_time_points=5,
            z_start=0,
            z_end=6,
            z_step=0.4,
        ),
        Simple=IMDAParams(
            num_time_points=2,
            z_start=0,
            z_end=2,
            z_step=0.1,
        ),
        Detailed=IMDAParams(
            num_time_points=10,
            z_start=0,
            z_end=12,
            z_step=0.2,
        ),
    )


@dataclass
class MDA:
    save_path: Path = Path().home() / ".avidaq" / "mda_presets.json"

    @property
    def paramsets(self) -> MDAPresets | dict[str, IMDAParams]:
        if not self.save_path.exists():
            print(f"{self.save_path} does not exist. Creating it.")
            self.save_path.touch()
            self.save_path.write_text(json.dumps(get_mda_presets(), indent=2))

        with open(self.save_path, "r+") as f:
            try:
                paramsets = json.load(f)
            except json.JSONDecodeError as e:
                print(e)
                paramsets = get_mda_presets()
                f.write(json.dumps(paramsets, indent=2))
            return paramsets

    @paramsets.setter
    def paramsets(self, paramsets: dict[str, IMDAParams]) -> None:
        with open(self.save_path, "w") as f:
            f.write(json.dumps(paramsets, indent=2))


if __name__ == "__main__":
    mda = MDA()
    print(mda.paramsets)



from __future__ import annotations

from pathlib import Path

import pytest

from avidaq._mda import get_mda_presets


def test_empty_json_file() -> None:

    # delete the file
    try:
        Path("mda_presets.json").unlink()
    except (FileNotFoundError):
        pass
    Path("mda_presets.json").touch()

    with pytest.raises(ValueError):
        file_data = get_mda_presets()
        assert file_data == {}
